@@ -22,6 +22,7 @@
 #include <libintl.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/param.h>
 #include <sys/stat.h>
 
 #include "config.h"
@@ -34,18 +35,7 @@ static xmlXPathCompExprPtr models_xpath;
 static xmlXPathCompExprPtr layouts_xpath;
 static xmlXPathCompExprPtr option_groups_xpath;
 
-
-#define XKBCR_MODEL_PATH "/xkbConfigRegistry/modelList/model"
-#define XKBCR_LAYOUT_PATH "/xkbConfigRegistry/layoutList/layout"
-#define XKBCR_VARIANT_PATH XKBCR_LAYOUT_PATH "/variantList/variant"
-#define XKBCR_GROUP_PATH "/xkbConfigRegistry/optionList/group"
-#define XKBCR_OPTION_PATH XKBCR_GROUP_PATH "/option"
-
-#define XML_TAG_DESCR "description"
-#define XML_TAG_SHORT_DESCR "shortDescription"
-#define XML_TAG_VENDOR "vendor"
-
-// gettext domain for translations
+/* gettext domain for translations */
 #define XKB_DOMAIN "xkeyboard-config"
 
 enum {
@@ -53,9 +43,6 @@ enum {
 	PROP_ENGINE,
 };
 
-#define xkl_config_registry_is_initialized(config) \
-  ( xkl_config_registry_priv(config,xpath_context) != NULL )
-
 static gboolean
 xkl_xml_find_config_item_child(xmlNodePtr iptr, xmlNodePtr * ptr)
 {
@@ -88,6 +75,7 @@ xkl_find_nonlocalized_element(xmlNodePtr ptr, const gchar * tag_name)
 {
 	xmlNodePtr found_element = NULL;
 
+	/* Look through all siblings, trying to find a node with proper name */
 	while (ptr != NULL) {
 		char *node_name = (char *) ptr->name;
 		if (ptr->type != XML_TEXT_NODE) {
@@ -95,8 +83,10 @@ xkl_find_nonlocalized_element(xmlNodePtr ptr, const gchar * tag_name)
 
 			if (lang == NULL) {	/* No language specified */
 				if (!g_ascii_strcasecmp
-				    (node_name, tag_name))
+				    (node_name, tag_name)) {
 					found_element = ptr;
+					break;
+				}
 			}
 		}
 		ptr = ptr->next;
@@ -105,12 +95,60 @@ xkl_find_nonlocalized_element(xmlNodePtr ptr, const gchar * tag_name)
 }
 
 static gboolean
+xkl_item_populate_optional_array(XklConfigItem * item, xmlNodePtr ptr,
+				 const gchar list_tag[],
+				 const gchar element_tag[],
+				 const gchar property_name[])
+{
+	xmlNodePtr top_list_element =
+	    xkl_find_nonlocalized_element(ptr, list_tag), element_ptr;
+	gint n_elements, idx;
+	gchar **elements = NULL;
+
+	if (top_list_element == NULL || top_list_element->children == NULL)
+		return FALSE;
+
+	n_elements = 0;
+
+	/* First, count countries */
+	element_ptr = top_list_element->children;
+	while (NULL !=
+	       (element_ptr =
+		xkl_find_nonlocalized_element(element_ptr, element_tag))) {
+		n_elements++;
+		element_ptr = element_ptr->next;
+	}
+
+	if (n_elements == 0)
+		return FALSE;
+
+	elements = g_new0(gchar *, n_elements + 1);
+	/* Then, actually, populate the list */
+	element_ptr = top_list_element->children;
+	for (idx = 0;
+	     NULL != (element_ptr =
+		      xkl_find_nonlocalized_element
+		      (element_ptr, element_tag));
+	     element_ptr = element_ptr->next, idx++) {
+		elements[idx] =
+		    g_strdup((const char *) element_ptr->children->
+			     content);
+	}
+
+	g_object_set_data_full(G_OBJECT(item),
+			       property_name, elements, (GDestroyNotify)
+			       g_strfreev);
+	return TRUE;
+}
+
+gboolean
 xkl_read_config_item(XklConfigRegistry * config, xmlNodePtr iptr,
 		     XklConfigItem * item)
 {
 	xmlNodePtr name_element, ptr;
 	xmlNodePtr desc_element = NULL, short_desc_element =
 	    NULL, vendor_element = NULL;
+
 	gchar *vendor = NULL;
 
 	*item->name = 0;
@@ -118,6 +156,8 @@ xkl_read_config_item(XklConfigRegistry * config, xmlNodePtr iptr,
 	*item->description = 0;
 
 	g_object_set_data(G_OBJECT(item), XCI_PROP_VENDOR, NULL);
+	g_object_set_data(G_OBJECT(item), XCI_PROP_COUNTRY_LIST, NULL);
+	g_object_set_data(G_OBJECT(item), XCI_PROP_LANGUAGE_LIST, NULL);
 
 	if (!xkl_xml_find_config_item_child(iptr, &ptr))
 		return FALSE;
@@ -135,18 +175,6 @@ xkl_read_config_item(XklConfigRegistry * config, xmlNodePtr iptr,
 	vendor_element =
 	    xkl_find_nonlocalized_element(ptr, XML_TAG_VENDOR);
 
-	if (vendor_element != NULL && vendor_element->children != NULL) {
-		vendor =
-		    g_strdup((const char *) vendor_element->children->
-			     content);
-		g_object_set_data_full(G_OBJECT(item), XCI_PROP_VENDOR,
-				       vendor, g_free);
-	}
-
-	/*
-	 * Actually, here we should have some code to find 
-	 * the correct localized description...
-	 */
 	if (name_element != NULL && name_element->children != NULL)
 		strncat(item->name,
 			(char *) name_element->children->content,
@@ -156,17 +184,34 @@ xkl_read_config_item(XklConfigRegistry * config, xmlNodePtr iptr,
 	    && short_desc_element->children != NULL) {
 		strncat(item->short_description,
 			dgettext(XKB_DOMAIN,
-				 (const char *) short_desc_element->
-				 children->content),
+				 (const char *)
+				 short_desc_element->children->content),
 			XKL_MAX_CI_SHORT_DESC_LENGTH - 1);
 	}
 
 	if (desc_element != NULL && desc_element->children != NULL) {
 		strncat(item->description,
 			dgettext(XKB_DOMAIN,
-				 (const char *) desc_element->children->
-				 content), XKL_MAX_CI_DESC_LENGTH - 1);
+				 (const char *) desc_element->
+				 children->content),
+			XKL_MAX_CI_DESC_LENGTH - 1);
 	}
+
+	if (vendor_element != NULL && vendor_element->children != NULL) {
+		vendor =
+		    g_strdup((const char *) vendor_element->
+			     children->content);
+		g_object_set_data_full(G_OBJECT(item), XCI_PROP_VENDOR,
+				       vendor, g_free);
+	}
+
+	xkl_item_populate_optional_array(item, ptr, XML_TAG_COUNTRY_LIST,
+					 XML_TAG_ISO3166ID,
+					 XCI_PROP_COUNTRY_LIST);
+	xkl_item_populate_optional_array(item, ptr, XML_TAG_LANGUAGE_LIST,
+					 XML_TAG_ISO639ID,
+					 XCI_PROP_LANGUAGE_LIST);
+
 	return TRUE;
 }
 
@@ -190,7 +235,7 @@ xkl_config_registry_foreach_in_nodeset(XklConfigRegistry * config,
 	}
 }
 
-static void
+void
 xkl_config_registry_foreach_in_xpath(XklConfigRegistry * config,
 				     xmlXPathCompExprPtr
 				     xpath_comp_expr,
@@ -206,14 +251,13 @@ xkl_config_registry_foreach_in_xpath(XklConfigRegistry * config,
 					 (config, xpath_context));
 	if (xpath_obj != NULL) {
 		xkl_config_registry_foreach_in_nodeset(config,
-						       xpath_obj->
-						       nodesetval,
+						       xpath_obj->nodesetval,
 						       func, data);
 		xmlXPathFreeObject(xpath_obj);
 	}
 }
 
-static void
+void
 xkl_config_registry_foreach_in_xpath_with_param(XklConfigRegistry
 						* config,
 						const gchar *
@@ -228,14 +272,13 @@ xkl_config_registry_foreach_in_xpath_with_param(XklConfigRegistry
 
 	if (!xkl_config_registry_is_initialized(config))
 		return;
-	snprintf(xpath_expr, sizeof xpath_expr, format, value);
+	g_snprintf(xpath_expr, sizeof xpath_expr, format, value);
 	xpath_obj = xmlXPathEval((unsigned char *) xpath_expr,
 				 xkl_config_registry_priv(config,
 							  xpath_context));
 	if (xpath_obj != NULL) {
 		xkl_config_registry_foreach_in_nodeset(config,
-						       xpath_obj->
-						       nodesetval,
+						       xpath_obj->nodesetval,
 						       func, data);
 		xmlXPathFreeObject(xpath_obj);
 	}
@@ -256,7 +299,8 @@ xkl_config_registry_find_object(XklConfigRegistry * config,
 	if (!xkl_config_registry_is_initialized(config))
 		return FALSE;
 
-	snprintf(xpath_expr, sizeof xpath_expr, format, arg1, pitem->name);
+	g_snprintf(xpath_expr, sizeof xpath_expr, format, arg1,
+		   pitem->name);
 	xpath_obj =
 	    xmlXPathEval((unsigned char *) xpath_expr,
 			 xkl_config_registry_priv(config, xpath_context));
@@ -435,15 +479,13 @@ xkl_config_registry_load_from_file(XklConfigRegistry * config,
 				   const gchar * file_name)
 {
 	xmlParserCtxtPtr ctxt = xmlNewParserCtxt();
-	xmlSAXHandler *saxh = g_new0(xmlSAXHandler, 1);
 	xkl_debug(100, "Loading XML registry from file %s\n", file_name);
 
 	/* Filter out all unneeded languages! */
-	xmlSAX2InitDefaultSAXHandler(saxh, TRUE);
-	saxh->startElementNs = xkl_xml_sax_start_element_ns;
-	saxh->endElementNs = xkl_xml_sax_end_element_ns;
-	saxh->characters = xkl_xml_sax_characters;
-	ctxt->sax = saxh;
+	xmlSAX2InitDefaultSAXHandler(ctxt->sax, TRUE);
+	ctxt->sax->startElementNs = xkl_xml_sax_start_element_ns;
+	ctxt->sax->endElementNs = xkl_xml_sax_end_element_ns;
+	ctxt->sax->characters = xkl_xml_sax_characters;
 
 	xkl_config_registry_priv(config, doc) =
 	    xmlCtxtReadFile(ctxt, file_name, NULL, XML_PARSE_NOBLANKS);
@@ -461,6 +503,30 @@ xkl_config_registry_load_from_file(XklConfigRegistry * config,
 	return TRUE;
 }
 
+gboolean
+xkl_config_registry_load_helper(XklConfigRegistry * config,
+				const char default_ruleset[],
+				const char base_dir[])
+{
+	struct stat stat_buf;
+	gchar file_name[MAXPATHLEN] = "";
+	XklEngine *engine = xkl_config_registry_get_engine(config);
+	gchar *rf = xkl_engine_get_ruleset_name(engine, default_ruleset);
+
+	if (rf == NULL || rf[0] == '\0')
+		return FALSE;
+
+	g_snprintf(file_name, sizeof file_name, "%s/%s.xml", base_dir, rf);
+
+	if (stat(file_name, &stat_buf) != 0) {
+		xkl_debug(0, "Missing registry file %s\n", file_name);
+		xkl_last_error_message = "Missing registry file";
+		return FALSE;
+	}
+
+	return xkl_config_registry_load_from_file(config, file_name);
+}
+
 void
 xkl_config_registry_free(XklConfigRegistry * config)
 {
@@ -822,6 +888,8 @@ xkl_config_registry_class_init(XklConfigRegistryClass * klass)
 	object_class->set_property = xkl_config_registry_set_property;
 	object_class->get_property = xkl_config_registry_get_property;
 
+	bind_textdomain_codeset(XKB_DOMAIN, "UTF-8");
+
 	engine_param_spec = g_param_spec_object("engine",
 						"Engine",
 						"XklEngine",
