@@ -396,6 +396,11 @@ extern void xkl_config_registry_free(XklConfigRegistry * config);
 extern gchar *xkl_locale_from_utf8(XklConfigRegistry * config,
 				   const gchar * utf8string);
 
+extern gboolean xkl_config_registry_load_helper(XklConfigRegistry * config,
+						const char
+						default_ruleset[],
+						const char base_dir[]);
+
 #define XKLAVIER_STATE_PROP_LENGTH 2
 
 /* taken from XFree86 maprules.c */
@@ -408,9 +413,44 @@ extern gchar *xkl_locale_from_utf8(XklConfigRegistry * config,
 #define xkl_engine_get_display(engine) (xkl_engine_priv(engine,display))
 #define xkl_engine_vcall(engine,func)  (*(engine)->priv->func)
 
+#define xkl_config_registry_is_initialized(config) \
+  ( xkl_config_registry_priv(config,xpath_context) != NULL )
+
 #define xkl_config_registry_priv(config,member)  (config)->priv->member
 #define xkl_config_registry_get_engine(config) ((config)->priv->engine)
 
+#define XKBCR_MODEL_PATH "/xkbConfigRegistry/modelList/model"
+#define XKBCR_LAYOUT_PATH "/xkbConfigRegistry/layoutList/layout"
+#define XKBCR_VARIANT_PATH XKBCR_LAYOUT_PATH "/variantList/variant"
+#define XKBCR_GROUP_PATH "/xkbConfigRegistry/optionList/group"
+#define XKBCR_OPTION_PATH XKBCR_GROUP_PATH "/option"
+
+#define XML_TAG_DESCR "description"
+#define XML_TAG_SHORT_DESCR "shortDescription"
+#define XML_TAG_VENDOR "vendor"
+#define XML_TAG_COUNTRY_LIST "countryList"
+#define XML_TAG_LANGUAGE_LIST "languageList"
+#define XML_TAG_ISO3166ID "iso3166Id"
+#define XML_TAG_ISO639ID "iso639Id"
+
+extern void
+xkl_config_registry_foreach_in_xpath_with_param(XklConfigRegistry * config,
+						const gchar * format,
+						const gchar * value,
+						ConfigItemProcessFunc func,
+						gpointer data);
+
+extern void xkl_config_registry_foreach_in_xpath(XklConfigRegistry *
+						 config,
+						 xmlXPathCompExprPtr
+						 xpath_comp_expr,
+						 ConfigItemProcessFunc
+						 func, gpointer data);
+
+extern gboolean xkl_read_config_item(XklConfigRegistry * config,
+				     xmlNodePtr iptr,
+				     XklConfigItem * item);
+
 extern gint xkl_debug_level;
 
 extern const gchar *xkl_last_error_message;
