@@ -46,6 +46,10 @@ xkl_engine_get_window_title(XklEngine * engine, Window w)
 	unsigned long nitems, rest;
 	unsigned char *prop;
 
+	if ((w == xkl_engine_priv(engine, root_window))
+	    || (w == PointerRoot))
+		return g_strdup("ROOT");
+
 	if (Success ==
 	    XGetWindowProperty(xkl_engine_get_display(engine), w,
 			       xkl_engine_priv(engine, atoms)[WM_NAME], 0L,
@@ -117,7 +121,7 @@ xkl_get_debug_window_title(XklEngine * engine, Window win)
 	if (win != (Window) NULL) {
 		name = xkl_engine_get_window_title(engine, win);
 		if (name != NULL) {
-			snprintf(sname, sizeof(sname), "%.32s", name);
+			g_snprintf(sname, sizeof(sname), "%.32s", name);
 			g_free(name);
 		}
 	}
