@@ -41,14 +41,35 @@ print_usage()
 	printf("         -l3 - listen to track the keyboard state\n");
 }
 
-void
-state_changed(XklEngine * engine, XklEngineStateChange type, gint new_group,
-	      gboolean restore)
+static void
+state_changed(XklEngine * engine, XklEngineStateChange type,
+	      gint new_group, gboolean restore)
 {
 	xkl_debug(0, "State changed: %d,%d,%d\n", type, new_group,
 		  restore);
 }
 
+static void
+config_changed(XklEngine * engine)
+{
+	const gchar **gn;
+	gint gt;
+	gint i;
+
+	xkl_debug(0, "Config changed\n");
+
+	gn = xkl_engine_get_groups_names(engine);
+	gt = xkl_engine_get_num_groups(engine);
+	for (i = 0; i < gt; i++)
+		xkl_debug(0, "group[%d]: [%s]\n", i, gn[i]);
+}
+
+static void
+new_device(XklEngine * engine)
+{
+	xkl_debug(0, "New device attached!\n");
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -105,7 +126,7 @@ main(int argc, char *argv[])
 	if (engine != NULL) {
 		XklConfigRec *current_config;
 		XklConfigRegistry *config;
-		const gchar** names;
+		const gchar **names;
 
 		xkl_debug(0, "Xklavier initialized\n");
 		config = xkl_config_registry_get_instance(engine);
@@ -117,21 +138,25 @@ main(int argc, char *argv[])
 
 		names = xkl_engine_get_groups_names(engine);
 		while (names != NULL && *names != NULL && **names != 0)
-			xkl_debug(0,"Group: [%s]\n", *names++);
+			xkl_debug(0, "Group: [%s]\n", *names++);
 
 		names = xkl_engine_get_indicators_names(engine);
 		while (names != NULL && *names != NULL && **names != 0)
-			xkl_debug(0,"Indicator: [%s]\n", *names++);
+			xkl_debug(0, "Indicator: [%s]\n", *names++);
 
 		g_signal_connect(engine, "X-state-changed",
 				 G_CALLBACK(state_changed), NULL);
+		g_signal_connect(engine, "X-config-changed",
+				 G_CALLBACK(config_changed), NULL);
+		g_signal_connect(engine, "X-new-device",
+				 G_CALLBACK(new_device), NULL);
 
 		xkl_debug(0, "Now, listening: %X...\n", listener_type);
 		xkl_engine_start_listen(engine, listener_type);
 
 		while (1) {
 			XNextEvent(dpy, &ev.core);
-			if (xkl_engine_filter_events(engine, &ev.core))
+			if (!xkl_engine_filter_events(engine, &ev.core))
 				xkl_debug(200, "Unknown event %d\n",
 					  ev.type);
 		}
